/*
 * Complex Bessel functions
 *
 * 2016-09-12 Kouichi Hirabayashi
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <complex.h>

#define COMPLEX double complex

extern void zbesi_(double*,double*,double*,int*,int*,double*,double*,int*,int*);
extern void zbesj_(double*,double*,double*,int*,int*,double*,double*,int*,int*);
extern void zbesk_(double*,double*,double*,int*,int*,double*,double*,int*,int*);
extern void zbesy_(double*,double*,double*,int*,int*,double*,double*,int*,double*,double*,int*);
extern void zbesh_(double*,double*,double*,int*,int*,int*,double*,double*,int*,int*);
extern void zairy_(double*,double*,int*,int*,double*,double*,int*,int*);
extern void zbiry_(double*,double*,int*,int*,double*,double*,int*,int*);

static void cbeserr(char *f, int ierr)
{
  char *s;
  switch (ierr) {
	case 1: s = "input error - no computation"; break;
	case 2: s = "overflow - no computation"; break;
	case 3: s = "cabs(z) large produce less than half of machine accurracy"; break;
	case 4: s = "cabs(z) too large - no computation"; break;
	case 5: s = "algorithm termination condition not met - no computation"; break;
  }
  printf("%s(): %s !\n", f, s);
}

COMPLEX In(int n, COMPLEX z){
  int kode=1;
  int N=1;
  double nu=(double)n;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  zbesi_(&zr,&zi,&nu,&kode,&N,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("In", ierr);
  res=cyr+I*cyi;
  return res;
}

COMPLEX Jn(int n, COMPLEX z){
  int kode=1;
  int N=1;
  double nu=(double)n;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  zbesj_(&zr,&zi,&nu,&kode,&N,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("Jn", ierr);
  res=cyr+I*cyi;
  return res;
}

COMPLEX Yn(int n, COMPLEX z){
  int kode=1;
  int N=1;
  double nu=(double)n;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi,cwrkr,cwrki;
  COMPLEX res;
  if (cabs(z) == 0.0)
	return -HUGE_VAL + 0.0 * I;
  zbesy_(&zr,&zi,&nu,&kode,&N,&cyr,&cyi,&nz,&cwrkr,&cwrki,&ierr);
  if (zi == 0.0 && zr >= 0.0)
	cyi = 0.0;
  if(ierr!=0)
	cbeserr("Yn", ierr);
  res=cyr+I*cyi;
  return res;
}

COMPLEX Kn(int n, COMPLEX z){
  int kode=1;
  int N=1;
  double nu=(double)n;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  if (cabs(z) == 0.0)
	return -HUGE_VAL + 0.0 * I;
  zbesk_(&zr,&zi,&nu,&kode,&N,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("Kn", ierr);
  res=cyr+I*cyi;
  return res;
}

// (d/dx)Jn(n, z)

COMPLEX Jnd(int n, COMPLEX z)
{
  if (n == 0)
	return -Jn(1, z);
  return (Jn(n - 1, z) - Jn(n + 1, z)) * 0.5;
//  return Jn(n - 1, z) - (COMPLEX)n / z * Jn(n, z);
}

// (d/dx)In(n, z)

COMPLEX Ind(int n, COMPLEX z)
{
  if (n == 0)
	return In(1, z);
  return (In(n - 1, z) + In(n + 1, z)) * 0.5;
//  return In(n - 1, z) - (COMPLEX)n / z * In(n, z);
}

COMPLEX Knd(int n, COMPLEX z)
{
  if(n == 0)
	return -Kn(1, z);
  return (Kn(n - 1, z) + Kn(n + 1, z)) * -0.5;
//  return -Kn(n - 1, z) - (COMPLEX)n / z * Kn(n, z);
}


COMPLEX H1(int n, COMPLEX z){
  int kode=1;
  int kind=1;
  int N=1;
  double nu=(double)n;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  if (cabs(z) == 0.0)
	return 1.0 + -I * HUGE_VAL;
  zbesh_(&zr,&zi,&nu,&kode,&kind,&N,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("H1", ierr);
  res=cyr+I*cyi;
  return res;
}

// (d/dx)H1(n, z)

COMPLEX H1d(int n, COMPLEX z)
{
  if (n == 0)
	return -H1(1, z);
  return (H1(n - 1, z) - H1(n + 1, z)) * 0.5;
//  return H1(n - 1, z) - (COMPLEX)n / z * H1(n, z);
}

COMPLEX H2(int n, COMPLEX z){
  int kode=1;
  int kind=2;
  int N=1;
  double nu=(double)n;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  if (cabs(z) == 0.0)
	return 1.0 + I * HUGE_VAL;
  zbesh_(&zr,&zi,&nu,&kode,&kind,&N,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("H2", ierr);
  res=cyr+I*cyi;
  return res;
}

// (d/dx)H2(n, z)

COMPLEX H2d(int n, COMPLEX z)
{
  if (n == 0)
	return -H2(1, z);
  return (H2(n - 1, z) - H2(n + 1, z)) * 0.5;
//  return H2(n - 1, z) - (COMPLEX)n / z * H2(n, z);
}

COMPLEX Ai(COMPLEX z){
  int kode=1;
  int id=0;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  zairy_(&zr,&zi,&id,&kode,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("Ai", ierr);
  res=cyr+I*cyi;
  return res;
}

COMPLEX Bi(COMPLEX z){
  int kode=1;
  int id=0;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  zbiry_(&zr,&zi,&id,&kode,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("Bi", ierr);
  res=cyr+I*cyi;
  return res;
}

COMPLEX Aid(COMPLEX z){
  int kode=1;
  int id=1;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  zairy_(&zr,&zi,&id,&kode,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("Aid", ierr);
  res=cyr+I*cyi;
  return res;
}

COMPLEX Bid(COMPLEX z){
  int kode=1;
  int id=1;
  double zr=creal(z);
  double zi=cimag(z);
  int nz,ierr;
  double cyr,cyi;
  COMPLEX res;
  zbiry_(&zr,&zi,&id,&kode,&cyr,&cyi,&nz,&ierr);
  if(ierr!=0)
	cbeserr("Bid", ierr);
  res=cyr+I*cyi;
  return res;
}

